@@ -2,16 +2,17 @@
 //!
 //! This module provides a prober for Darcs repositories. It can detect
 //! darcs repositories but does not provide any additional functionality.
-use crate::controldir::Prober;
 use pyo3::exceptions::PyModuleNotFoundError;
 use pyo3::prelude::*;
 
+/// A prober for Darcs repositories.
 pub struct DarcsProber(PyObject);
 
 impl DarcsProber {
+    /// Create a new Darcs prober instance.
     pub fn new() -> Option<Self> {
         Python::with_gil(|py| {
-            let m = match py.import_bound("breezy.plugins.darcs") {
+            let m = match py.import("breezy.plugins.darcs") {
                 Ok(m) => m,
                 Err(e) => {
                     if e.is_instance_of::<PyModuleNotFoundError>(py) {
@@ -23,25 +24,39 @@ impl DarcsProber {
                 }
             };
             let prober = m.getattr("DarcsProber").expect("Failed to get DarcsProber");
-            Some(Self(prober.to_object(py)))
+            Some(Self(prober.unbind()))
         })
     }
 }
 
 impl FromPyObject<'_> for DarcsProber {
     fn extract_bound(obj: &Bound<PyAny>) -> PyResult<Self> {
-        Ok(Self(obj.to_object(obj.py())))
+        Ok(Self(obj.clone().unbind()))
     }
 }
 
-impl ToPyObject for DarcsProber {
+impl<'py> IntoPyObject<'py> for DarcsProber {
+    type Target = PyAny;
+    type Output = Bound<'py, Self::Target>;
+    type Error = std::convert::Infallible;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        Ok(self.0.into_bound(py))
+    }
+}
+
+impl std::fmt::Debug for DarcsProber {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        f.write_fmt(format_args!("DarcsProber({:?})", self.0))
+    }
+}
+
+impl crate::controldir::PyProber for DarcsProber {
     fn to_object(&self, py: Python) -> PyObject {
-        self.0.to_object(py)
+        self.0.clone_ref(py)
     }
 }
 
-impl Prober for DarcsProber {}
-
 #[cfg(test)]
 mod tests {
     use super::*;
