@@ -1,17 +1,21 @@
 //! Fast and efficient detection of files that have been modified in a directory tree.
-use crate::tree::WorkingTree;
+use crate::workingtree::{GenericWorkingTree, WorkingTree};
 use dirty_tracker::DirtyTracker;
 pub use dirty_tracker::State;
 
+/// Tracks changes in a working tree.
+///
+/// This struct provides functionality to efficiently detect which files have been
+/// modified in a working tree.
 pub struct DirtyTreeTracker {
     tracker: DirtyTracker,
-    tree: WorkingTree,
+    tree: GenericWorkingTree,
     base: std::path::PathBuf,
 }
 
 impl DirtyTreeTracker {
     /// Create a new DirtyTreeTracker for the given WorkingTree.
-    pub fn new(tree: WorkingTree) -> Self {
+    pub fn new(tree: GenericWorkingTree) -> Self {
         let base = tree.basedir();
         let tracker = DirtyTracker::new(&base).unwrap();
         Self {
@@ -21,7 +25,10 @@ impl DirtyTreeTracker {
         }
     }
 
-    pub fn new_in_subpath(tree: WorkingTree, subpath: &std::path::Path) -> Self {
+    /// Create a new DirtyTreeTracker for a specific subpath within a WorkingTree.
+    ///
+    /// This allows tracking changes only within a specific subdirectory of the working tree.
+    pub fn new_in_subpath(tree: GenericWorkingTree, subpath: &std::path::Path) -> Self {
         let base = tree.basedir();
         let tracker = DirtyTracker::new(&base.join(subpath)).unwrap();
         Self {
