@@ -1,6 +1,7 @@
 //! Error handling for the Breezy Python bindings
 use crate::transform::RawConflict;
 use pyo3::import_exception;
+use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::PyErr;
 use url::Url;
@@ -49,7 +50,9 @@ import_exception!(breezy.errors, BadHttpRequest);
 import_exception!(breezy.errors, TransportNotPossible);
 import_exception!(breezy.errors, IncompatibleFormat);
 import_exception!(breezy.errors, NoSuchRevision);
+import_exception!(breezy.errors, RevisionNotPresent);
 import_exception!(breezy.forge, NoSuchProject);
+import_exception!(breezy.errors, ObjectNotLocked);
 import_exception!(breezy.plugins.gitlab.forge, ForkingDisabled);
 import_exception!(breezy.plugins.gitlab.forge, GitLabConflict);
 import_exception!(breezy.plugins.gitlab.forge, ProjectCreationTimeout);
@@ -61,96 +64,174 @@ import_exception!(breezy.errors, NoRoundtrippingSupport);
 import_exception!(breezy.inter, NoCompatibleInter);
 
 lazy_static::lazy_static! {
-    // Only present in breezy << 4.0
+    /// Static reference to the BreezyConnectionError class, if it exists.
+    ///
+    /// This is only present in Breezy versions before 4.0.
     pub static ref BreezyConnectionError: Option<PyObject> = { Python::with_gil(|py| {
-        let m = py.import_bound("breezy.errors").unwrap();
-        m.getattr("ConnectionError").ok().map(|x| x.to_object(py))
+        let m = py.import("breezy.errors").unwrap();
+        m.getattr("ConnectionError").ok().map(|x| x.unbind())
     })
 };
 }
 
+/// Error type for the Breezy Rust wrapper.
+///
+/// This enum represents the various errors that can occur when using the Breezy
+/// API through this Rust wrapper. Each variant corresponds to a specific error
+/// condition, with many mapping directly to Python exceptions from the Breezy
+/// library.
 #[derive(Debug)]
 pub enum Error {
+    /// A Python error that doesn't map to a specific variant.
     Other(PyErr),
+    /// An unknown format was encountered.
     UnknownFormat(String),
+    /// The specified path is not a branch.
     NotBranchError(String, Option<String>),
+    /// The repository format doesn't support colocated branches.
     NoColocatedBranchSupport,
+    /// A required dependency is not present.
     DependencyNotPresent(String, String),
+    /// Permission was denied when accessing a path.
     PermissionDenied(std::path::PathBuf, Option<String>),
+    /// The specified protocol is not supported.
     UnsupportedProtocol(String, Option<String>),
+    /// A redirect could not be followed.
     UnusableRedirect(String, String, String),
+    /// A connection error occurred.
     ConnectionError(String),
+    /// The specified URL is invalid.
     InvalidURL(String, Option<String>),
+    /// An error occurred during transport.
     TransportError(String),
+    /// The specified format is not supported.
     UnsupportedFormat(String),
+    /// The specified version control system is not supported.
     UnsupportedVcs(String),
+    /// An error occurred when interacting with a remote Git repository.
     RemoteGitError(String),
+    /// A read operation did not complete (partial read).
     IncompleteRead(Vec<u8>, Option<usize>),
+    /// An error occurred with line endings in a file.
     LineEndingError(String),
+    /// An invalid HTTP response was received.
     InvalidHttpResponse(
         String,
         String,
         Option<String>,
         std::collections::HashMap<String, String>,
     ),
+    /// A control directory already exists at the specified path.
     AlreadyControlDir(std::path::PathBuf),
+    /// A branch already exists at the specified path.
     AlreadyBranch(std::path::PathBuf),
+    /// The branches have diverged.
     DivergedBranches,
+    /// The workspace has uncommitted changes.
     WorkspaceDirty(std::path::PathBuf),
+    /// The specified file does not exist.
     NoSuchFile(std::path::PathBuf),
+    /// The commit would not change anything.
     PointlessCommit,
+    /// No user identity has been configured.
     NoWhoami,
+    /// The specified tag does not exist.
     NoSuchTag(String),
+    /// The specified tag already exists.
     TagAlreadyExists(String),
+    /// A socket error occurred.
     Socket(std::io::Error),
+    /// Login to the forge is required.
     ForgeLoginRequired,
+    /// The specified forge is not supported.
     UnsupportedForge(url::Url),
+    /// A project already exists on the forge.
     ForgeProjectExists(String),
+    /// A merge proposal already exists.
     MergeProposalExists(url::Url, Option<url::Url>),
+    /// The operation is not supported.
     UnsupportedOperation(String, String),
+    /// A protected branch hook declined the push.
     ProtectedBranchHookDeclined(String),
+    /// No repository is present at the specified location.
     NoRepositoryPresent,
+    /// Failed to acquire a lock.
     LockFailed(String),
+    /// A file already exists at the specified path.
     FileExists(std::path::PathBuf, Option<String>),
+    /// Lock contention occurred.
     LockContention(String, String),
+    /// The requested operation is not implemented.
     NotImplemented,
+    /// The specified revision is not in the tree.
     NoSuchRevisionInTree(crate::RevisionId),
+    /// A nested tree is missing at the specified path.
     MissingNestedTree(std::path::PathBuf),
-    /// Failed to delete transform temporary directory
+    /// Failed to delete transform temporary directory.
     ImmortalLimbo(std::path::PathBuf),
+    /// The transform is malformed.
     MalformedTransform(Vec<RawConflict>),
+    /// Failed to rename a file during a transform operation.
     TransformRenameFailed(
         std::path::PathBuf,
         std::path::PathBuf,
         String,
         std::io::Error,
     ),
+    /// An unexpected HTTP status code was received.
     UnexpectedHttpStatus {
+        /// The URL that was requested.
         url: url::Url,
+        /// The HTTP status code that was received.
         code: u16,
+        /// Additional information about the error.
         extra: Option<String>,
+        /// The HTTP headers that were received.
         headers: std::collections::HashMap<String, String>,
     },
+    /// A timeout occurred.
     Timeout,
+    /// A bad HTTP request was made.
     BadHttpRequest(Url, String),
+    /// The transport is not possible.
     TransportNotPossible(String),
+    /// The format is not compatible.
     IncompatibleFormat(String, String),
+    /// The specified revision does not exist.
     NoSuchRevision(crate::RevisionId),
+    /// The specified revision is not present in the repository.
+    RevisionNotPresent(crate::RevisionId),
+    /// The specified project does not exist.
     NoSuchProject(String),
+    /// Forking is disabled for the specified project.
     ForkingDisabled(String),
+    /// The project creation timed out.
     ProjectCreationTimeout(String, chrono::Duration),
+    /// A conflict occurred in GitLab.
     GitLabConflict(String),
+    /// The source branch is not derived from the target branch.
     SourceNotDerivedFromTarget,
+    /// A loop was detected in branch references.
     BranchReferenceLoop,
+    /// A redirect was requested.
     RedirectRequested {
+        /// The source URL.
         source: url::Url,
+        /// The target URL.
         target: url::Url,
+        /// Whether the redirect is permanent.
         is_permanent: bool,
     },
+    /// There are conflicts in the tree.
     ConflictsInTree,
+    /// The operation does not support roundtripping.
     NoRoundtrippingSupport,
+    /// No compatible inter-repository implementation was found.
     NoCompatibleInter,
+    /// The resource is read-only.
     ReadOnly,
+    /// An object that needs to be locked is not locked.
+    ObjectNotLocked(String),
 }
 
 impl From<url::ParseError> for Error {
@@ -284,6 +365,7 @@ impl std::fmt::Display for Error {
                 write!(f, "Incompatible format: {} is not compatible with {}", a, b)
             }
             Self::NoSuchRevision(rev) => write!(f, "No such revision: {}", rev),
+            Self::RevisionNotPresent(rev) => write!(f, "Revision not present: {}", rev),
             Self::NoSuchProject(p) => write!(f, "No such project: {}", p),
             Self::ForkingDisabled(p) => write!(f, "Forking disabled: {}", p),
             Self::ProjectCreationTimeout(p, t) => {
@@ -296,6 +378,7 @@ impl std::fmt::Display for Error {
             Self::NoRoundtrippingSupport => write!(f, "No roundtripping support"),
             Self::NoCompatibleInter => write!(f, "No compatible inter"),
             Self::ReadOnly => write!(f, "Read-only"),
+            Self::ObjectNotLocked(msg) => write!(f, "Object not locked: {}", msg),
             Self::RedirectRequested {
                 source,
                 target,
@@ -317,7 +400,7 @@ impl From<PyErr> for Error {
     fn from(err: PyErr) -> Self {
         pyo3::import_exception!(socket, error);
         pyo3::Python::with_gil(|py| {
-            let value = err.value_bound(py);
+            let value = err.value(py);
             if err.is_instance_of::<UnknownFormatError>(py) {
                 Error::UnknownFormat(value.getattr("format").unwrap().extract().unwrap())
             } else if err.is_instance_of::<NotBranchError>(py) {
@@ -539,7 +622,7 @@ impl From<PyErr> for Error {
                         format
                     } else {
                         format
-                            .call_method0("get_format_string")
+                            .call_method0(intern!(py, "get_format_string"))
                             .unwrap()
                             .extract()
                             .unwrap()
@@ -548,7 +631,7 @@ impl From<PyErr> for Error {
                         controldir
                     } else {
                         controldir
-                            .call_method0("get_format_string")
+                            .call_method0(intern!(py, "get_format_string"))
                             .unwrap()
                             .extract()
                             .unwrap()
@@ -556,6 +639,8 @@ impl From<PyErr> for Error {
                 )
             } else if err.is_instance_of::<NoSuchRevision>(py) {
                 Error::NoSuchRevision(value.getattr("revision").unwrap().extract().unwrap())
+            } else if err.is_instance_of::<RevisionNotPresent>(py) {
+                Error::RevisionNotPresent(value.getattr("revision_id").unwrap().extract().unwrap())
             } else if err.is_instance_of::<NoSuchProject>(py) {
                 Error::NoSuchProject(value.getattr("project").unwrap().extract().unwrap())
             } else if err.is_instance_of::<ForkingDisabled>(py) {
@@ -574,7 +659,12 @@ impl From<PyErr> for Error {
             } else if BreezyConnectionError
                 .as_ref()
                 .and_then(|cls| {
-                    Python::with_gil(|py| Some(err.is_instance_bound(py, cls.bind(py))))
+                    Python::with_gil(|py| {
+                        Some(
+                            err.is_instance_of::<PyAny>(py)
+                                && err.value(py).is_instance(cls.bind(py)).unwrap(),
+                        )
+                    })
                 })
                 .unwrap_or(false)
             {
@@ -615,6 +705,8 @@ impl From<PyErr> for Error {
                 Error::TransportError(value.getattr("msg").unwrap().extract().unwrap())
             } else if err.is_instance_of::<BranchReferenceLoop>(py) {
                 Error::BranchReferenceLoop
+            } else if err.is_instance_of::<ObjectNotLocked>(py) {
+                Error::ObjectNotLocked(err.to_string())
             } else {
                 if std::env::var("BRZ_ERROR").is_ok() {
                     // Print backtrace
@@ -650,7 +742,7 @@ impl From<Error> for PyErr {
             Error::UnsupportedVcs(s) => UnsupportedVcs::new_err((s,)),
             Error::RemoteGitError(e) => RemoteGitError::new_err((e,)),
             Error::IncompleteRead(partial, expected) => Python::with_gil(|py| {
-                let bytes = pyo3::types::PyBytes::new_bound(py, partial.as_slice());
+                let bytes = pyo3::types::PyBytes::new(py, partial.as_slice());
                 IncompleteRead::new_err((bytes.unbind(), expected))
             }),
             Error::LineEndingError(e) => LineEndingError::new_err((e,)),
@@ -707,7 +799,9 @@ impl From<Error> for PyErr {
             }
             Error::ImmortalLimbo(p) => ImmortalLimbo::new_err((p.to_string_lossy().to_string(),)),
             Error::MalformedTransform(conflicts) => {
-                MalformedTransform::new_err((Python::with_gil(|py| conflicts.to_object(py)),))
+                MalformedTransform::new_err((Python::with_gil(|py| {
+                    conflicts.into_pyobject(py).unwrap().unbind()
+                }),))
             }
             Error::TransformRenameFailed(from_path, to_path, why, error) => {
                 TransformRenameFailed::new_err((
@@ -732,6 +826,7 @@ impl From<Error> for PyErr {
             Error::NoSuchRevision(rev) => {
                 Python::with_gil(|py| NoSuchRevision::new_err((py.None(), rev.to_string())))
             }
+            Error::RevisionNotPresent(rev) => RevisionNotPresent::new_err((rev.to_string(),)),
             Error::NoSuchProject(p) => NoSuchProject::new_err((p,)),
             Error::ForkingDisabled(p) => ForkingDisabled::new_err((p,)),
             Error::ProjectCreationTimeout(p, t) => ProjectCreationTimeout::new_err((p, t)),
@@ -740,6 +835,7 @@ impl From<Error> for PyErr {
             Error::SourceNotDerivedFromTarget => SourceNotDerivedFromTarget::new_err(()),
             Error::BranchReferenceLoop => BranchReferenceLoop::new_err(()),
             Error::ReadOnly => Python::with_gil(|py| ReadOnlyError::new_err((py.None(),))),
+            Error::ObjectNotLocked(msg) => ObjectNotLocked::new_err((msg,)),
             Error::RedirectRequested {
                 source,
                 target,
