@@ -3,32 +3,67 @@ use pyo3::prelude::*;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
 #[derive(Clone, PartialEq, Eq, Ord, PartialOrd, Hash)]
+/// Represents a unique identifier for a revision in a version control system.
+///
+/// RevisionId is typically a string in UTF-8 encoding, but is stored as bytes
+/// to efficiently handle all possible revision formats across different VCS systems.
 pub struct RevisionId(Vec<u8>);
 
 impl std::fmt::Debug for RevisionId {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let s = String::from_utf8(self.0.clone()).unwrap();
+        let s = String::from_utf8_lossy(&self.0);
         write!(f, "{}", s)
     }
 }
 
 impl RevisionId {
+    /// Get the raw bytes of the revision ID.
+    ///
+    /// # Returns
+    ///
+    /// A slice of the underlying bytes
     pub fn as_bytes(&self) -> &[u8] {
         &self.0
     }
 
+    /// Check if this revision ID is the null revision.
+    ///
+    /// # Returns
+    ///
+    /// `true` if this is the null revision, `false` otherwise
     pub fn is_null(&self) -> bool {
         self.0 == NULL_REVISION
     }
 
+    /// Check if this revision ID is a reserved revision.
+    ///
+    /// Reserved revision IDs start with a colon character.
+    ///
+    /// # Returns
+    ///
+    /// `true` if this is a reserved revision, `false` otherwise
     pub fn is_reserved(&self) -> bool {
         self.0.starts_with(b":")
     }
 
+    /// Create a new null revision ID.
+    ///
+    /// # Returns
+    ///
+    /// A new RevisionId representing the null revision
     pub fn null() -> Self {
         Self(NULL_REVISION.to_vec())
     }
 
+    /// Get the revision ID as a UTF-8 string.
+    ///
+    /// # Returns
+    ///
+    /// The revision ID as a string slice
+    ///
+    /// # Panics
+    ///
+    /// Panics if the revision ID is not valid UTF-8
     pub fn as_str(&self) -> &str {
         std::str::from_utf8(&self.0).unwrap()
     }
@@ -81,7 +116,7 @@ impl Serialize for RevisionId {
     where
         S: Serializer,
     {
-        serializer.serialize_str(String::from_utf8(self.0.clone()).unwrap().as_str())
+        serializer.serialize_str(&String::from_utf8_lossy(&self.0))
     }
 }
 
@@ -101,26 +136,35 @@ impl FromPyObject<'_> for RevisionId {
     }
 }
 
-impl ToPyObject for RevisionId {
-    fn to_object(&self, py: Python) -> PyObject {
-        pyo3::types::PyBytes::new_bound(py, &self.0).to_object(py)
+impl<'py> IntoPyObject<'py> for RevisionId {
+    type Target = PyAny;
+    type Output = Bound<'py, Self::Target>;
+    type Error = std::convert::Infallible;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        Ok(pyo3::types::PyBytes::new(py, &self.0).into_any())
     }
 }
 
-impl IntoPy<PyObject> for RevisionId {
-    fn into_py(self, py: Python) -> PyObject {
-        pyo3::types::PyBytes::new_bound(py, self.0.as_slice()).to_object(py)
-    }
-}
+// IntoPy is replaced by IntoPyObject in PyO3 0.25
+// The IntoPyObject implementation above handles the conversion
 
 impl std::fmt::Display for RevisionId {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let s = String::from_utf8(self.0.clone()).unwrap();
+        let s = String::from_utf8_lossy(&self.0);
         write!(f, "{}", s)
     }
 }
 
+/// Constant representing the "current" revision identifier.
+///
+/// This is used to refer to the current revision in working trees.
 pub const CURRENT_REVISION: &[u8] = b"current:";
+
+/// Constant representing the "null" revision identifier.
+///
+/// The null revision is used to represent the absence of a revision,
+/// such as the parent of the first commit in a repository.
 pub const NULL_REVISION: &[u8] = b"null:";
 
 #[cfg(test)]
