@@ -5,15 +5,33 @@ use std::fs;
 use std::path::PathBuf;
 use tempfile::TempDir;
 
+/// Environment for running Breezy tests.
+///
+/// This struct sets up a temporary environment for running Breezy tests,
+/// including temporary directories and environment variables, and cleans
+/// up after itself when dropped.
 pub struct TestEnv {
+    /// The temporary directory that contains all test files.
     pub temp_dir: TempDir,
+    /// The working directory where tests will run.
     pub working_dir: PathBuf,
+    /// The home directory for the test environment.
     pub home_dir: PathBuf,
+    /// The original working directory before the test environment was set up.
     pub old_cwd: PathBuf,
+    /// The original environment variables before the test environment was set up.
     pub old_env: HashMap<String, Option<String>>,
 }
 
 impl TestEnv {
+    /// Create a new testing environment.
+    ///
+    /// This sets up a temporary directory structure with a working directory
+    /// and home directory, and configures environment variables for Breezy.
+    ///
+    /// # Returns
+    ///
+    /// A new TestEnv instance
     pub fn new() -> Self {
         let temp_dir = TempDir::new().unwrap();
         let working_dir = temp_dir.path().join("test");
@@ -32,7 +50,7 @@ impl TestEnv {
         std::env::set_var("BRZ_EMAIL", brz_email);
         std::env::set_var("BRZ_HOME", &breezy_home);
         pyo3::Python::with_gil(|py| {
-            let os = py.import_bound("os").unwrap();
+            let os = py.import("os").unwrap();
             os.call_method1("chdir", (working_dir.to_str().unwrap(),))
                 .unwrap();
             let environ = os.getattr("environ").unwrap();
@@ -72,7 +90,7 @@ impl Drop for TestEnv {
                 std::env::remove_var(&key);
             }
             Python::with_gil(|py| {
-                let os = py.import_bound("os").unwrap();
+                let os = py.import("os").unwrap();
                 let environ = os.getattr("environ").unwrap();
                 if let Some(value) = value {
                     environ.set_item(key, value).unwrap();
@@ -116,7 +134,7 @@ mod tests {
         );
 
         Python::with_gil(|py| {
-            let os = py.import_bound("os").unwrap();
+            let os = py.import("os").unwrap();
             assert_eq!(
                 os.call_method0("getcwd")
                     .unwrap()
