@@ -1,13 +1,15 @@
 //! Tree transformation API.
-use crate::tree::{PathBuf, Tree};
+use crate::tree::{PathBuf, PyTree};
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::PyTuple;
 use pyo3::types::PyTupleMethods;
 
+/// A tree transform is used to apply a set of changes to a tree.
 pub struct TreeTransform(PyObject);
 
 #[derive(Clone)]
+/// Represents a change to a file or directory in a tree transformation.
 pub struct TreeChange {}
 
 impl From<PyObject> for TreeChange {
@@ -22,6 +24,7 @@ impl FromPyObject<'_> for TreeChange {
     }
 }
 
+/// Represents a conflict that occurs during a tree transformation.
 pub struct Conflict(PyObject);
 
 impl Clone for Conflict {
@@ -31,13 +34,14 @@ impl Clone for Conflict {
 }
 
 impl Conflict {
+    /// Get the file paths associated with this conflict.
     pub fn associated_filenames(&self) -> Result<Vec<PathBuf>, crate::error::Error> {
         let mut v: Vec<PathBuf> = vec![];
 
         Python::with_gil(|py| {
             let ret = self.0.getattr(py, "associated_filenames")?;
 
-            for item in ret.bind(py).iter()? {
+            for item in ret.bind(py).try_iter()? {
                 v.push(item?.extract()?);
             }
 
@@ -45,6 +49,7 @@ impl Conflict {
         })
     }
 
+    /// Get a human-readable description of this conflict.
     pub fn describe(&self) -> Result<String, crate::error::Error> {
         Python::with_gil(|py| {
             let ret = self.0.call_method0(py, "describe")?;
@@ -52,7 +57,8 @@ impl Conflict {
         })
     }
 
-    pub fn cleanup(&self, tree: &dyn Tree) -> Result<(), crate::error::Error> {
+    /// Clean up any temporary files created by this conflict.
+    pub fn cleanup<T: PyTree>(&self, tree: &T) -> Result<(), crate::error::Error> {
         Python::with_gil(|py| {
             self.0.call_method1(py, "cleanup", (tree.to_object(py),))?;
             Ok(())
@@ -60,11 +66,16 @@ impl Conflict {
     }
 }
 
+/// A tree that shows what a tree would look like after applying a transform.
 pub struct PreviewTree(PyObject);
 
-impl ToPyObject for PreviewTree {
-    fn to_object(&self, py: Python) -> PyObject {
-        self.0.to_object(py)
+impl<'py> IntoPyObject<'py> for PreviewTree {
+    type Target = PyAny;
+    type Output = Bound<'py, Self::Target>;
+    type Error = std::convert::Infallible;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        Ok(self.0.into_bound(py))
     }
 }
 
@@ -74,25 +85,33 @@ impl From<PyObject> for PreviewTree {
     }
 }
 
-impl Tree for PreviewTree {}
-
 impl TreeTransform {
+    fn to_object(&self) -> &PyObject {
+        &self.0
+    }
+
+    /// Get the underlying PyObject.
+    pub(crate) fn as_pyobject(&self) -> &PyObject {
+        &self.0
+    }
+    /// Apply the transform to the tree.
     pub fn finalize(&self) -> Result<(), crate::error::Error> {
         Python::with_gil(|py| {
-            self.to_object(py).call_method0(py, "finalize")?;
+            self.to_object().call_method0(py, "finalize")?;
             Ok(())
         })
     }
 
+    /// Iterate through the changes in this transform.
     pub fn iter_changes(
         &self,
     ) -> Result<Box<dyn Iterator<Item = TreeChange>>, crate::error::Error> {
         let mut v: Vec<TreeChange> = vec![];
 
         Python::with_gil(|py| {
-            let ret = self.to_object(py).call_method0(py, "iter_changes")?;
+            let ret = self.to_object().call_method0(py, "iter_changes")?;
 
-            for item in ret.bind(py).iter()? {
+            for item in ret.bind(py).try_iter()? {
                 v.push(item?.extract()?);
             }
 
@@ -100,13 +119,14 @@ impl TreeTransform {
         })
     }
 
+    /// Get a list of conflicts that would occur when applying this transform.
     pub fn cooked_conflicts(&self) -> Result<Vec<Conflict>, crate::error::Error> {
         let mut v: Vec<Conflict> = vec![];
 
         Python::with_gil(|py| {
-            let ret = self.to_object(py).getattr(py, "cooked_conflicts")?;
+            let ret = self.to_object().getattr(py, "cooked_conflicts")?;
 
-            for item in ret.bind(py).iter()? {
+            for item in ret.bind(py).try_iter()? {
                 v.push(Conflict(item?.into()));
             }
 
@@ -114,9 +134,10 @@ impl TreeTransform {
         })
     }
 
+    /// Get a preview tree showing what would happen if this transform was applied.
     pub fn get_preview_tree(&self) -> Result<PreviewTree, crate::error::Error> {
         Python::with_gil(|py| {
-            let ret = self.to_object(py).getattr(py, "preview_tree")?;
+            let ret = self.to_object().getattr(py, "preview_tree")?;
             Ok(PreviewTree(ret))
         })
     }
@@ -128,9 +149,13 @@ impl From<PyObject> for TreeTransform {
     }
 }
 
-impl ToPyObject for TreeTransform {
-    fn to_object(&self, py: Python) -> PyObject {
-        self.0.to_object(py)
+impl<'py> IntoPyObject<'py> for TreeTransform {
+    type Target = PyAny;
+    type Output = Bound<'py, Self::Target>;
+    type Error = std::convert::Infallible;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        Ok(self.0.into_bound(py))
     }
 }
 
@@ -141,6 +166,7 @@ impl FromPyObject<'_> for TreeTransform {
 }
 
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
+/// An identifier for a transformation operation.
 pub struct TransId(String);
 
 impl FromPyObject<'_> for TransId {
@@ -149,71 +175,87 @@ impl FromPyObject<'_> for TransId {
     }
 }
 
-impl ToPyObject for TransId {
-    fn to_object(&self, py: Python) -> PyObject {
-        self.0.to_object(py)
+impl<'py> IntoPyObject<'py> for TransId {
+    type Target = PyAny;
+    type Output = Bound<'py, Self::Target>;
+    type Error = std::convert::Infallible;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        Ok(self.0.into_pyobject(py)?.into_any())
     }
 }
 
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
+/// Enum representing different types of conflicts that can occur during transformation.
 pub enum RawConflict {
+    /// Conflict caused by trying to change executability of an unversioned file.
     UnversionedExecutability(TransId),
+    /// Conflict caused by trying to set executability on a non-file.
     NonFileExecutability(TransId),
+    /// Conflict caused by trying to overwrite an existing file with different content.
     Overwrite(TransId, String),
+    /// Conflict caused by a directory loop in the parent structure.
     ParentLoop(TransId),
+    /// Conflict caused by trying to version a file with an unversioned parent.
     UnversionedParent(TransId),
+    /// Conflict caused by trying to version a file without contents.
     VersioningNoContents(TransId),
+    /// Conflict caused by trying to version a file with an unsupported kind.
     VersioningBadKind(TransId),
+    /// Conflict caused by trying to add the same file path twice.
     Duplicate(TransId, TransId, String),
+    /// Conflict caused by a missing parent directory.
     MissingParent(TransId),
+    /// Conflict caused by a parent that is not a directory.
     NonDirectoryParent(TransId),
 }
 
-impl ToPyObject for RawConflict {
-    fn to_object(&self, py: Python) -> PyObject {
-        match self {
+impl<'py> IntoPyObject<'py> for RawConflict {
+    type Target = PyAny;
+    type Output = Bound<'py, Self::Target>;
+    type Error = std::convert::Infallible;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        Ok(match self {
             RawConflict::UnversionedExecutability(id) => {
-                PyTuple::new_bound(py, &[("unversioned executability", id.to_object(py))])
-                    .to_object(py)
+                PyTuple::new(py, [("unversioned executability", id)])
+                    .unwrap()
+                    .into_any()
             }
             RawConflict::NonFileExecutability(id) => {
-                PyTuple::new_bound(py, &[("non-file executability", id.to_object(py))])
-                    .to_object(py)
-            }
-            RawConflict::Overwrite(id, path) => {
-                PyTuple::new_bound(py, &[("overwrite", id.to_object(py), path.to_object(py))])
-                    .to_object(py)
+                PyTuple::new(py, [("non-file executability", id)])
+                    .unwrap()
+                    .into_any()
             }
+            RawConflict::Overwrite(id, path) => PyTuple::new(py, [("overwrite", id, path)])
+                .unwrap()
+                .into_any(),
             RawConflict::ParentLoop(id) => {
-                PyTuple::new_bound(py, &[("parent loop", id.to_object(py))]).to_object(py)
-            }
-            RawConflict::UnversionedParent(id) => {
-                PyTuple::new_bound(py, &[("unversioned parent", id.to_object(py))]).to_object(py)
+                PyTuple::new(py, [("parent loop", id)]).unwrap().into_any()
             }
+            RawConflict::UnversionedParent(id) => PyTuple::new(py, [("unversioned parent", id)])
+                .unwrap()
+                .into_any(),
             RawConflict::VersioningNoContents(id) => {
-                PyTuple::new_bound(py, &[("versioning no contents", id.to_object(py))])
-                    .to_object(py)
+                PyTuple::new(py, [("versioning no contents", id)])
+                    .unwrap()
+                    .into_any()
             }
-            RawConflict::VersioningBadKind(id) => {
-                PyTuple::new_bound(py, &[("versioning bad kind", id.to_object(py))]).to_object(py)
+            RawConflict::VersioningBadKind(id) => PyTuple::new(py, [("versioning bad kind", id)])
+                .unwrap()
+                .into_any(),
+            RawConflict::Duplicate(id1, id2, path) => {
+                PyTuple::new(py, [("duplicate", id1, id2, path)])
+                    .unwrap()
+                    .into_any()
             }
-            RawConflict::Duplicate(id1, id2, path) => PyTuple::new_bound(
-                py,
-                &[(
-                    "duplicate",
-                    id1.to_object(py),
-                    id2.to_object(py),
-                    path.to_object(py),
-                )],
-            )
-            .to_object(py),
-            RawConflict::MissingParent(id) => {
-                PyTuple::new_bound(py, &[("missing parent", id.to_object(py))]).to_object(py)
-            }
-            RawConflict::NonDirectoryParent(id) => {
-                PyTuple::new_bound(py, &[("non-directory parent", id.to_object(py))]).to_object(py)
-            }
-        }
+            RawConflict::MissingParent(id) => PyTuple::new(py, [("missing parent", id)])
+                .unwrap()
+                .into_any(),
+            RawConflict::NonDirectoryParent(id) => PyTuple::new(py, [("non-directory parent", id)])
+                .unwrap()
+                .into_any(),
+        })
     }
 }
 
