@@ -1,21 +1,38 @@
 //! Version information for the Breezy library.
 use pyo3::prelude::*;
 
+/// The release level of a version.
+///
+/// This enum represents the different stages of a software release.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub enum ReleaseLevel {
+    /// Development version.
     Dev,
+    /// Alpha version.
     Alpha,
+    /// Beta version.
     Beta,
+    /// Release candidate.
     Candidate,
+    /// Final release.
     Final,
 }
 
+/// Version information.
+///
+/// This struct represents a version number with major, minor, and micro components,
+/// a release level, and a serial number.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub struct Version {
+    /// The major version number.
     major: u32,
+    /// The minor version number.
     minor: u32,
+    /// The micro (patch) version number.
     micro: u32,
+    /// The release level.
     level: ReleaseLevel,
+    /// The serial number within the release level.
     serial: u32,
 }
 
@@ -43,11 +60,16 @@ impl std::fmt::Display for Version {
     }
 }
 
+/// Get the version of the Breezy library.
+///
+/// # Returns
+///
+/// The version of the Breezy library.
 pub fn version() -> Version {
     pyo3::prepare_freethreaded_python();
 
     Python::with_gil(|py| {
-        let m = py.import_bound("breezy").unwrap();
+        let m = py.import("breezy").unwrap();
 
         let version_info = m.getattr("version_info").unwrap();
 
